#' Paste simple JavaScript plot code
#' 
#' This function is similar to \code{rk.paste.JS}, but adds some code parts to its output which
#' are commonly used to generate plots with RKWard.
#' 
#' The contents of the \code{...} argument are evaluated by \code{rk.paste.JS} and encapsulated
#' between \code{if(full)\{rk.graph.on()\} try(\{} and \code{\}) if(full)\{rk.graph.off()\}}. If generic
#' plot options are supplied, their \code{"code.preprocess"} and \code{"code.calculate"} modifiers are
#' also automatically taken care of, so you only need to include \code{"code.printout"} inside of
#' \code{...}.
#'
#' @param ... The actual plot code, passed through to \code{rk.paste.JS}.
#' @param plotOpts A node object generated by \code{rk.XML.embed}, i.e. embedded plot options.
#' @param printoutObj An \code{rk.JS.var} object fetching the \code{"code.printout"} modifier of \code{plotOpts}
#'		(see examples below!).
#' @return A character string.
#' @export
#' @seealso
#'		\code{\link[rkwarddev:rk.paste.JS]{rk.paste.JS}}
#' @examples
#' tmp.var.selectVars <- rk.XML.varselector(label="Select data")
#' tmp.var.x <- rk.XML.varslot(label="My data", source=tmp.var.selectVars, required=TRUE)
#' # let this be the embedded generic plot options in yout plot dialog
#' tmp.plot.options <- rk.XML.embed(component="rkward::plot_options", button=TRUE, label="Generic plot options")
#' 
#' # now generate an rk.JS.var object to import the "printout" code from the embedded options
#' js.po.printout <- rk.JS.vars(tmp.plot.options, modifiers="code.printout", check.modifiers=FALSE)
#' 
#' # you can now generate the plot code using generic plot options
#' js.prnt <- rk.paste.JS.graph(
#'	echo("\t\tplot("),
#'	echo("\n\t\t\tx=", tmp.var.x),
#'	echo(js.po.printout),
#'	echo(")"),
#'	plotOpts=tmp.plot.options,
#'	printoutObj=js.po.printout)
#'
#' cat(js.prnt)
rk.paste.JS.graph <- function(..., plotOpts=NULL, printoutObj=NULL, level=2, indent.by="\t", empty.e=FALSE){

	plotOptsIndent <- paste(rep("\\t", level), collapse="")

	# define variables
	js.prnt <- rk.paste.JS(
		if(!is.null(plotOpts)){
			rk.paste.JS(
				rk.comment("in case there are generic plot options defined:"),
				js.po.preprocess <- rk.JS.vars(plotOpts, modifiers="code.preprocess", check.modifiers=FALSE),
				if(!is.null(printoutObj)){
					printoutObj
				} else {
					warning("rk.paste.JS.graph: you're using plot options, but 'printoutObj' is empty, is that intended?")
				},
				js.po.calculate <- rk.JS.vars(plotOpts, modifiers="code.calculate", check.modifiers=FALSE)
			)
		} else {},
		level=level, indent.by=indent.by, empty.e=empty.e
	)

	# graph.on() & begin try()
	js.prnt <- paste(js.prnt, rk.paste.JS(
		ite("full", echo("rk.graph.on()\n")),
		echo("\ttry({\n"),
		level=level, indent.by=indent.by, empty.e=empty.e
	), sep="\n\n")

	# plot options: preprocess
	js.prnt <- paste(js.prnt, rk.paste.JS(
		if(!is.null(plotOpts)){
			rk.paste.JS(
				rk.comment("insert any option-setting code that should be run before the actual plotting commands:"),
				id("\t\tprintIndentedUnlessEmpty(\"", plotOptsIndent, "\", ", js.po.preprocess, ", \"\\n\", \"\");")
			)
		} else {},
		level=level, indent.by=indent.by, empty.e=empty.e
	), sep="\n\n")

	# here comes the plot
	js.prnt <- paste(js.prnt, rk.paste.JS(
		rk.comment("the actual plot:"),
		rk.paste.JS(..., level=level, indent.by=indent.by, empty.e=empty.e),
		level=level, indent.by=indent.by, empty.e=empty.e
	), sep="\n\n")

	# plot options: postprocess
	js.prnt <- paste(js.prnt, rk.paste.JS(
		if(!is.null(plotOpts)){
			rk.paste.JS(
				rk.comment("insert any option-setting code that should be run after the actual plot:"),
				id("\t\tprintIndentedUnlessEmpty(\"", plotOptsIndent, "\", ", js.po.calculate, ", \"\\n\", \"\");")
			)
		} else {},
		level=level, indent.by=indent.by, empty.e=empty.e
	), sep="\n\n")

	# end try() & graph.off()
	js.prnt <- paste(js.prnt, rk.paste.JS(
		echo("\n\t})\n"),
		ite("full", echo("rk.graph.off()\n")),
		level=level, indent.by=indent.by, empty.e=empty.e
	), sep="\n\n")

	return(js.prnt)
}
